/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "usb_device.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "usbd_cdc_if.h"
#include "bd3491.h"
#include "message_parser.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */
#define clamp(v, min, max) (((long)(v) > (long)(max))?(max):((long)(v) < (long)(min))?min:v)
/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

#define OUTPUT_MAP(a) \
	a(5, SW3) \
	a(4, SW2) \
	a(3, SW5) \
	a(2, SW6) \
	a(1, SW4) \
	a(0, SW7)

#define OUTPUT_LED(a) \
	a(5, SW5) \
	a(4, SW2) \
	a(3, SW3) \
	a(2, SW6) \
	a(1, SW4) \
	a(0, SW7)

void set_input_led(int number, int value) {
	switch(number) {
#define CASE(i, name) case i: HAL_GPIO_WritePin(name ## _LED_GPIO_Port, name ## _LED_Pin, value); break;
	OUTPUT_LED(CASE)
#undef CASE
	default:
		break;
	}
}

int test_button_pressed() {
#define TEST_BUTTON(num, name) \
	if (GPIO_PIN_RESET == HAL_GPIO_ReadPin(name ## _GPIO_Port, name ## _Pin)) { \
		return num; \
	} else

	OUTPUT_MAP(TEST_BUTTON)
	{
		return -1;
	}

#undef TEST_BUTTON
}

struct ui_state ui_state = {
  .oldchannel = -1,
  .state = UI_STATE_NORMAL,
};

void set_input(uint8_t channel) {
	if (ui_state.state != UI_STATE_NORMAL || ui_state.oldchannel == channel || channel >= 6)
		return;

  set_input_led(ui_state.oldchannel, 0);	
  
  set_input_led(channel, 1);

  // invert bd inputs to match the numbers on the case
  bd_set_input(&hi2c1, 5-channel);

	ui_state.oldchannel = channel;

	printf("C%d\n", channel);
}

int get_input() {
  return ui_state.oldchannel;
}

// only mute if the ui allows to 
void set_mute(uint8_t mute) {
  if(UI_STATE_MUTE == ui_state.state && mute)
    // no state change required
    return;

  // figure out the state transition  
  if (UI_STATE_NORMAL == ui_state.state && mute) {
    ui_state.state = UI_STATE_MUTE;
    HAL_GPIO_WritePin(USER_LED_GPIO_Port, USER_LED_Pin, 1);
    set_input_led(ui_state.oldchannel, 0);
    bd_set_input(&hi2c1, BD_INPUT_MUTE);
  } 
  else if (UI_STATE_MUTE == ui_state.state && !mute) 
  {
    ui_state.state = UI_STATE_NORMAL;
    // hack to make reactivation of the last channel easier
    uint8_t old_input = ui_state.oldchannel;
    ui_state.oldchannel = -1;

    set_input(old_input);
    HAL_GPIO_WritePin(USER_LED_GPIO_Port, USER_LED_Pin, 0); 
  }
}

int is_muted() {
  return ui_state.state == UI_STATE_MUTE;
}

void set_short(uint8_t do_short) {
  // test for possible state changes
  if (UI_STATE_NORMAL == ui_state.state && do_short) 
  {
    ui_state.state = UI_STATE_SHORT;
    bd_set_input(&hi2c1, BD_INPUT_ALL);

    // visualize that all inputs are shorted
    for (int i = 0; i < 6; ++i) 
      set_input_led(i, 1);
  } 
  else if (UI_STATE_SHORT == ui_state.state && !do_short) 
  {
    ui_state.state = UI_STATE_NORMAL;
    
    for (int i = 0; i < 6; ++i) 
      set_input_led(i, 0);

    // reactivate currently selected input
    uint8_t old_input = ui_state.oldchannel;
    ui_state.oldchannel = -1;
    set_input(old_input);
  }
}

int is_shorted() {
  return ui_state.state == UI_STATE_SHORT;
}

void set_attenuation(uint8_t left, uint8_t right) {
	ui_state.latt = bd_set_attenuation(&hi2c1, 0, left);
  ui_state.ratt = bd_set_attenuation(&hi2c1, 1, right);
}



/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */
  

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_I2C1_Init();
  MX_USB_DEVICE_Init();
  /* USER CODE BEGIN 2 */
  HAL_GPIO_TogglePin(USER_LED_GPIO_Port, USER_LED_Pin);
  HAL_Delay(100);
  HAL_GPIO_TogglePin(USER_LED_GPIO_Port, USER_LED_Pin);
  uint8_t input = 0;

  // reset set input
  set_input(input);
  set_attenuation(0,0);
  int count = 0;
  int button_timeout = 0;
  
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

	  // handle button inputs
	  input = test_button_pressed();
	  if (6 > input) {
		  set_input(input);
      
      if (100 < ++count) {
        count = 0;
        button_timeout++;
      }

      //printf("%d\n", count);

      if (1000 < button_timeout) {
        // long press, toggle mute
        set_mute(!is_muted());
        printf("M%d\n", is_muted());
        button_timeout = 0;
      }

	  } else {
      count = 0;
      button_timeout = 0;
    }

	  // parse serial commands
	  parse_buffer();
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USB;
  PeriphClkInit.UsbClockSelection = RCC_USBCLKSOURCE_PLL_DIV1_5;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
